@@ -36,7 +36,7 @@
 #include <ogg/ogg.h>
 #include <string.h>
 
-#include "gstoggelements.h"
+#include "gstogg.h"
 #include "gstoggstream.h"
 
 GST_DEBUG_CATEGORY_STATIC (gst_ogg_parse_debug);
@@ -81,12 +81,34 @@ struct _GstOggParseClass
   GstElementClass parent_class;
 };
 
-
+static void gst_ogg_parse_base_init (gpointer g_class);
+static void gst_ogg_parse_class_init (GstOggParseClass * klass);
+static void gst_ogg_parse_init (GstOggParse * ogg);
 static GstElementClass *parent_class = NULL;
-G_DEFINE_TYPE (GstOggParse, gst_ogg_parse, GST_TYPE_ELEMENT);
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (oggparse, "oggparse", GST_RANK_NONE,
-    GST_TYPE_OGG_PARSE, GST_DEBUG_CATEGORY_INIT (gst_ogg_parse_debug,
-        "oggparse", 0, "ogg parser"));
+
+static GType
+gst_ogg_parse_get_type (void)
+{
+  static GType ogg_parse_type = 0;
+
+  if (!ogg_parse_type) {
+    static const GTypeInfo ogg_parse_info = {
+      sizeof (GstOggParseClass),
+      gst_ogg_parse_base_init,
+      NULL,
+      (GClassInitFunc) gst_ogg_parse_class_init,
+      NULL,
+      NULL,
+      sizeof (GstOggParse),
+      0,
+      (GInstanceInitFunc) gst_ogg_parse_init,
+    };
+
+    ogg_parse_type = g_type_register_static (GST_TYPE_ELEMENT, "GstOggParse",
+        &ogg_parse_info, 0);
+  }
+  return ogg_parse_type;
+}
 
 static void
 free_stream (GstOggStream * stream)
@@ -95,7 +117,7 @@ free_stream (GstOggStream * stream)
   g_list_foreach (stream->unknown_pages, (GFunc) gst_mini_object_unref, NULL);
   g_list_foreach (stream->stored_buffers, (GFunc) gst_mini_object_unref, NULL);
 
-  g_free (stream);
+  g_slice_free (GstOggStream, stream);
 }
 
 static void
@@ -118,7 +140,7 @@ gst_ogg_parse_new_stream (GstOggParse * parser, ogg_page * page)
 
   GST_DEBUG_OBJECT (parser, "creating new stream %08x", serialno);
 
-  stream = g_new0 (GstOggStream, 1);
+  stream = g_slice_new0 (GstOggStream);
 
   stream->serialno = serialno;
   stream->in_headers = 1;
@@ -126,19 +148,16 @@ gst_ogg_parse_new_stream (GstOggParse * parser, ogg_page * page)
   if (ogg_stream_init (&stream->stream, serialno) != 0) {
     GST_ERROR ("Could not initialize ogg_stream struct for serial %08x.",
         serialno);
-    goto failure;
+    return NULL;
   }
 
-  if (ogg_stream_pagein (&stream->stream, page) != 0)
-    goto failure;
+  /* FIXME check return */
+  ogg_stream_pagein (&stream->stream, page);
 
+  /* FIXME check return */
   ret = ogg_stream_packetout (&stream->stream, &packet);
   if (ret == 1) {
-    if (!gst_ogg_stream_setup_map (stream, &packet)) {
-      GST_ERROR ("Could not setup map for ogg packet.");
-      goto failure;
-    }
-
+    gst_ogg_stream_setup_map (stream, &packet);
     if (stream->is_video) {
       parser->video_stream = stream;
     }
@@ -147,10 +166,6 @@ gst_ogg_parse_new_stream (GstOggParse * parser, ogg_page * page)
   parser->oggstreams = g_slist_append (parser->oggstreams, stream);
 
   return stream;
-
-failure:
-  free_stream (stream);
-  return NULL;
 }
 
 static GstOggStream *
@@ -201,20 +216,26 @@ static GstFlowReturn gst_ogg_parse_chain (GstPad * pad, GstObject * parent,
     GstBuffer * buffer);
 
 static void
-gst_ogg_parse_class_init (GstOggParseClass * klass)
+gst_ogg_parse_base_init (gpointer g_class)
 {
-  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
-  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_static_metadata (gstelement_class,
+  gst_element_class_set_static_metadata (element_class,
       "Ogg parser", "Codec/Parser",
       "parse ogg streams into pages (info about ogg: http://xiph.org)",
       "Michael Smith <msmith@fluendo.com>");
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &ogg_parse_sink_template_factory);
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &ogg_parse_src_template_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&ogg_parse_sink_template_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&ogg_parse_src_template_factory));
+}
+
+static void
+gst_ogg_parse_class_init (GstOggParseClass * klass)
+{
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   parent_class = g_type_class_peek_parent (klass);
 
@@ -456,10 +477,6 @@ gst_ogg_parse_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
         }
 
         stream = gst_ogg_parse_new_stream (ogg, &page);
-        if (!stream) {
-          GST_LOG_OBJECT (ogg, "Incorrect page");
-          goto failure;
-        }
 
         ogg->last_page_not_bos = FALSE;
 
@@ -719,3 +736,12 @@ gst_ogg_parse_change_state (GstElement * element, GstStateChange transition)
   }
   return result;
 }
+
+gboolean
+gst_ogg_parse_plugin_init (GstPlugin * plugin)
+{
+  GST_DEBUG_CATEGORY_INIT (gst_ogg_parse_debug, "oggparse", 0, "ogg parser");
+
+  return gst_element_register (plugin, "oggparse", GST_RANK_NONE,
+      GST_TYPE_OGG_PARSE);
+}
