@@ -20,6 +20,10 @@
  * Boston, MA 02110-1301, USA.
  */
 
+#ifndef __GST_AUDIO_AUDIO_H__
+#include <gst/audio/audio.h>
+#endif
+
 #ifndef __GST_AUDIO_SRC_H__
 #define __GST_AUDIO_SRC_H__
 
@@ -60,8 +64,8 @@ struct _GstAudioSrc {
  * @prepare: configure device with format
  * @unprepare: undo the configuration
  * @close: close the device
- * @read: read samples to the audio device
- * @delay: the number of samples queued in the device
+ * @read: read samples from the audio device
+ * @delay: the number of frames queued in the device
  * @reset: unblock a read to the device and reset.
  *
  * #GstAudioSrc class. Override the vmethod to implement
@@ -83,7 +87,7 @@ struct _GstAudioSrcClass {
   /* read samples from the device */
   guint    (*read)      (GstAudioSrc *src, gpointer data, guint length,
       GstClockTime *timestamp);
-  /* get number of samples queued in the device */
+  /* get number of frames queued in the device */
   guint    (*delay)     (GstAudioSrc *src);
   /* reset the audio device, unblock from a write */
   void     (*reset)     (GstAudioSrc *src);
