@@ -13,9 +13,13 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 
 #include <glib.h>
 #include <math.h>
@@ -65,7 +69,7 @@ struct _GstFFTF64
 };
 
 /**
- * gst_fft_f64_new:
+ * gst_fft_f64_new: (skip)
  * @len: Length of the FFT in the time domain
  * @inverse: %TRUE if the #GstFFTF64 instance should be used for the inverse FFT
  *
