@@ -23,11 +23,11 @@
 
 #include <gst/gst.h>
 
+G_BEGIN_DECLS
+
 #include <gst/pbutils/pbutils-enumtypes.h>
 #include <gst/pbutils/gstdiscoverer.h>
 
-G_BEGIN_DECLS
-
 /**
  * GstEncodingProfile:
  *
@@ -43,8 +43,6 @@ G_BEGIN_DECLS
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_ENCODING_PROFILE))
 typedef struct _GstEncodingProfile GstEncodingProfile;
 typedef struct _GstEncodingProfileClass GstEncodingProfileClass;
-
-GST_PBUTILS_API
 GType gst_encoding_profile_get_type (void);
 
 
@@ -62,8 +60,6 @@ GType gst_encoding_profile_get_type (void);
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_ENCODING_CONTAINER_PROFILE))
 typedef struct _GstEncodingContainerProfile GstEncodingContainerProfile;
 typedef struct _GstEncodingContainerProfileClass GstEncodingContainerProfileClass;
-
-GST_PBUTILS_API
 GType gst_encoding_container_profile_get_type (void);
 
 
@@ -81,8 +77,6 @@ GType gst_encoding_container_profile_get_type (void);
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_ENCODING_VIDEO_PROFILE))
 typedef struct _GstEncodingVideoProfile GstEncodingVideoProfile;
 typedef struct _GstEncodingVideoProfileClass GstEncodingVideoProfileClass;
-
-GST_PBUTILS_API
 GType gst_encoding_video_profile_get_type (void);
 
 
@@ -100,8 +94,6 @@ GType gst_encoding_video_profile_get_type (void);
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_ENCODING_AUDIO_PROFILE))
 typedef struct _GstEncodingAudioProfile GstEncodingAudioProfile;
 typedef struct _GstEncodingAudioProfileClass GstEncodingAudioProfileClass;
-
-GST_PBUTILS_API
 GType gst_encoding_audio_profile_get_type (void);
 
 
@@ -124,169 +116,79 @@ GType gst_encoding_audio_profile_get_type (void);
  */
 #define gst_encoding_profile_ref(profile) (g_object_ref ((GObject*) profile))
 
-GST_PBUTILS_API
 const gchar *   gst_encoding_profile_get_name           (GstEncodingProfile *profile);
-
-GST_PBUTILS_API
 void            gst_encoding_profile_set_name           (GstEncodingProfile *profile,
                                                          const gchar *name);
-
-GST_PBUTILS_API
 const gchar *   gst_encoding_profile_get_description    (GstEncodingProfile *profile);
-
-GST_PBUTILS_API
 void            gst_encoding_profile_set_description    (GstEncodingProfile *profile,
                                                          const gchar *description);
-
-GST_PBUTILS_API
-GstCaps *       gst_encoding_profile_get_format         (GstEncodingProfile *profile) G_GNUC_WARN_UNUSED_RESULT;
-
-GST_PBUTILS_API
+GstCaps *       gst_encoding_profile_get_format         (GstEncodingProfile *profile);
 void            gst_encoding_profile_set_format         (GstEncodingProfile *profile,
                                                          GstCaps *format);
-
-GST_PBUTILS_API
 gboolean  gst_encoding_profile_get_allow_dynamic_output (GstEncodingProfile *profile);
-
-GST_PBUTILS_API
 void      gst_encoding_profile_set_allow_dynamic_output (GstEncodingProfile *profile,
                                                          gboolean allow_dynamic_output);
-
-GST_PBUTILS_API
-gboolean  gst_encoding_profile_get_single_segment       (GstEncodingProfile *profile);
-
-GST_PBUTILS_API
-void      gst_encoding_profile_set_single_segment       (GstEncodingProfile *profile,
-                                                         gboolean single_segment);
-
-GST_PBUTILS_API
 const gchar *   gst_encoding_profile_get_preset         (GstEncodingProfile *profile);
-
-GST_PBUTILS_API
 const gchar *   gst_encoding_profile_get_preset_name    (GstEncodingProfile *profile);
-
-GST_PBUTILS_API
 void            gst_encoding_profile_set_preset         (GstEncodingProfile *profile,
                                                          const gchar *preset);
-
-GST_PBUTILS_API
 guint           gst_encoding_profile_get_presence       (GstEncodingProfile *profile);
-
-GST_PBUTILS_API
 void            gst_encoding_profile_set_presence       (GstEncodingProfile *profile,
                                                          guint presence);
-
-GST_PBUTILS_API
 void            gst_encoding_profile_set_preset_name    (GstEncodingProfile * profile,
                                                          const gchar * preset_name);
-
-GST_PBUTILS_API
-GstCaps *       gst_encoding_profile_get_restriction    (GstEncodingProfile *profile) G_GNUC_WARN_UNUSED_RESULT;
-
-GST_PBUTILS_API
+GstCaps *       gst_encoding_profile_get_restriction    (GstEncodingProfile *profile);
 void            gst_encoding_profile_set_restriction    (GstEncodingProfile *profile,
                                                          GstCaps *restriction);
 
-GST_PBUTILS_API
 gboolean        gst_encoding_profile_is_equal           (GstEncodingProfile *a,
                                                          GstEncodingProfile *b);
-
-GST_PBUTILS_API
-GstCaps *       gst_encoding_profile_get_input_caps     (GstEncodingProfile *profile) G_GNUC_WARN_UNUSED_RESULT;
-
-GST_PBUTILS_API
+GstCaps *       gst_encoding_profile_get_input_caps     (GstEncodingProfile *profile);
 const gchar *   gst_encoding_profile_get_type_nick      (GstEncodingProfile *profile);
 
-GST_PBUTILS_API
 const gchar *   gst_encoding_profile_get_file_extension (GstEncodingProfile * profile);
 
-GST_PBUTILS_API
 GstEncodingProfile * gst_encoding_profile_find (const gchar *targetname,
                                                 const gchar *profilename,
                                                 const gchar *category);
 
-GST_PBUTILS_API
 gboolean        gst_encoding_profile_is_enabled        (GstEncodingProfile *profile);
-
-GST_PBUTILS_API
 void            gst_encoding_profile_set_enabled       (GstEncodingProfile *profile,
                                                          gboolean enabled);
 /* GstEncodingContainerProfile API */
-
-GST_PBUTILS_API
 gboolean        gst_encoding_container_profile_add_profile       (GstEncodingContainerProfile *container,
                                                                   GstEncodingProfile *profile);
-
-GST_PBUTILS_API
 gboolean        gst_encoding_container_profile_contains_profile  (GstEncodingContainerProfile * container,
                                                                   GstEncodingProfile *profile);
-
-GST_PBUTILS_API
 const GList *   gst_encoding_container_profile_get_profiles      (GstEncodingContainerProfile *profile);
 
 
-GST_PBUTILS_API
 GstEncodingContainerProfile *  gst_encoding_container_profile_new (const gchar *name,
                                                                    const gchar *description,
                                                                    GstCaps *format,
-                                                                   const gchar *preset) G_GNUC_WARN_UNUSED_RESULT;
+                                                                   const gchar *preset);
 
 
-/* Individual stream encodingprofile API */
-
-GST_PBUTILS_API
+/* Invidual stream encodingprofile API */
 GstEncodingVideoProfile * gst_encoding_video_profile_new (GstCaps *format,
                                                           const gchar *preset,
                                                           GstCaps *restriction,
-                                                          guint presence) G_GNUC_WARN_UNUSED_RESULT;
-
-GST_PBUTILS_API
+                                                          guint presence);
 GstEncodingAudioProfile * gst_encoding_audio_profile_new (GstCaps *format,
                                                           const gchar *preset,
                                                           GstCaps *restriction,
-                                                          guint presence) G_GNUC_WARN_UNUSED_RESULT;
+                                                          guint presence);
 
-GST_PBUTILS_API
 guint    gst_encoding_video_profile_get_pass              (GstEncodingVideoProfile *prof);
-
-GST_PBUTILS_API
 gboolean gst_encoding_video_profile_get_variableframerate (GstEncodingVideoProfile *prof);
 
-GST_PBUTILS_API
 void     gst_encoding_video_profile_set_pass              (GstEncodingVideoProfile *prof,
                                                            guint pass);
-
-GST_PBUTILS_API
 void     gst_encoding_video_profile_set_variableframerate (GstEncodingVideoProfile *prof,
                                                            gboolean variableframerate);
 
-GST_PBUTILS_API
 GstEncodingProfile * gst_encoding_profile_from_discoverer (GstDiscovererInfo *info);
 
-GST_PBUTILS_API
-GstEncodingProfile * gst_encoding_profile_copy            (GstEncodingProfile *self) G_GNUC_WARN_UNUSED_RESULT;
-
-GST_PBUTILS_API
-GstEncodingProfile * gst_encoding_profile_from_string     (const gchar *string);
-
-GST_PBUTILS_API
-gchar * gst_encoding_profile_to_string     (GstEncodingProfile *profile) G_GNUC_WARN_UNUSED_RESULT;
-
-GST_PBUTILS_API
-void gst_encoding_profile_set_element_properties          (GstEncodingProfile *self,
-                                                           GstStructure *element_properties);
-
-GST_PBUTILS_API
-GstStructure *gst_encoding_profile_get_element_properties (GstEncodingProfile *self) G_GNUC_WARN_UNUSED_RESULT;
-
-G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstEncodingAudioProfile, gst_object_unref)
-
-G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstEncodingContainerProfile, gst_object_unref)
-
-G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstEncodingProfile, gst_object_unref)
-
-G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstEncodingVideoProfile, gst_object_unref)
-
 G_END_DECLS
 
 #endif /* __GST_PROFILE_H__ */
