@@ -44,8 +44,12 @@ struct _GstRTPBasePayloadPrivate
   gboolean perfect_rtptime;
   gint notified_first_timestamp;
 
+  gboolean pt_set;
+
   guint64 base_offset;
   gint64 base_rtime;
+  guint64 base_rtime_hz;
+  guint64 running_time;
 
   gint64 prop_max_ptime;
   gint64 caps_max_ptime;
@@ -54,6 +58,8 @@ struct _GstRTPBasePayloadPrivate
 
   gboolean delay_segment;
   GstEvent *pending_segment;
+
+  GstCaps *subclass_srccaps;
 };
 
 /* RTPBasePayload signals and args */
@@ -78,6 +84,7 @@ enum
 #define DEFAULT_MIN_PTIME               0
 #define DEFAULT_PERFECT_RTPTIME         TRUE
 #define DEFAULT_PTIME_MULTIPLE          0
+#define DEFAULT_RUNNING_TIME            GST_CLOCK_TIME_NONE
 
 enum
 {
@@ -93,6 +100,7 @@ enum
   PROP_SEQNUM,
   PROP_PERFECT_RTPTIME,
   PROP_PTIME_MULTIPLE,
+  PROP_STATS,
   PROP_LAST
 };
 
@@ -108,6 +116,10 @@ static gboolean gst_rtp_base_payload_sink_event_default (GstRTPBasePayload *
     rtpbasepayload, GstEvent * event);
 static gboolean gst_rtp_base_payload_sink_event (GstPad * pad,
     GstObject * parent, GstEvent * event);
+static gboolean gst_rtp_base_payload_src_event_default (GstRTPBasePayload *
+    rtpbasepayload, GstEvent * event);
+static gboolean gst_rtp_base_payload_src_event (GstPad * pad,
+    GstObject * parent, GstEvent * event);
 static gboolean gst_rtp_base_payload_query_default (GstRTPBasePayload *
     rtpbasepayload, GstPad * pad, GstQuery * query);
 static gboolean gst_rtp_base_payload_query (GstPad * pad, GstObject * parent,
@@ -123,6 +135,9 @@ static void gst_rtp_base_payload_get_property (GObject * object, guint prop_id,
 static GstStateChangeReturn gst_rtp_base_payload_change_state (GstElement *
     element, GstStateChange transition);
 
+static gboolean gst_rtp_base_payload_negotiate (GstRTPBasePayload * payload);
+
+
 static GstElementClass *parent_class = NULL;
 
 GType
@@ -175,7 +190,7 @@ gst_rtp_base_payload_class_init (GstRTPBasePayloadClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PT,
       g_param_spec_uint ("pt", "payload type",
-          "The payload type of the packets", 0, 0x80, DEFAULT_PT,
+          "The payload type of the packets", 0, 0x7f, DEFAULT_PT,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SSRC,
       g_param_spec_uint ("ssrc", "SSRC",
@@ -197,7 +212,7 @@ gst_rtp_base_payload_class_init (GstRTPBasePayloadClass * klass)
           -1, G_MAXINT64, DEFAULT_MAX_PTIME,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
-   * GstRTPBaseAudioPayload:min-ptime:
+   * GstRTPBasePayload:min-ptime:
    *
    * Minimum duration of the packet data in ns (can't go above MTU)
    **/
@@ -217,19 +232,30 @@ gst_rtp_base_payload_class_init (GstRTPBasePayloadClass * klass)
           0, G_MAXUINT16, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GstRTPBaseAudioPayload:perfect-rtptime:
+   * GstRTPBasePayload:perfect-rtptime:
    *
-   * Try to use the offset fields to generate perfect RTP timestamps. when this
-   * option is disabled, RTP timestamps are generated from the GStreamer
-   * timestamps, which could result in RTP timestamps that don't increment with
-   * the amount of data in the packet.
+   * Try to use the offset fields to generate perfect RTP timestamps. When this
+   * option is disabled, RTP timestamps are generated from GST_BUFFER_PTS of
+   * each payloaded buffer. The PTSes of buffers may not necessarily increment
+   * with the amount of data in each input buffer, consider e.g. the case where
+   * the buffer arrives from a network which means that the PTS is unrelated to
+   * the amount of data. Because the RTP timestamps are generated from
+   * GST_BUFFER_PTS this can result in RTP timestamps that also don't increment
+   * with the amount of data in the payloaded packet. To circumvent this it is
+   * possible to set the perfect rtptime option enabled. When this option is
+   * enabled the payloader will increment the RTP timestamps based on
+   * GST_BUFFER_OFFSET which relates to the amount of data in each packet
+   * rather than the GST_BUFFER_PTS of each buffer and therefore the RTP
+   * timestamps will more closely correlate with the amount of data in each
+   * buffer. Currently GstRTPBasePayload is limited to handling perfect RTP
+   * timestamps for audio streams.
    */
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PERFECT_RTPTIME,
       g_param_spec_boolean ("perfect-rtptime", "Perfect RTP Time",
           "Generate perfect RTP timestamps when possible",
           DEFAULT_PERFECT_RTPTIME, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
-   * GstRTPBaseAudioPayload:ptime-multiple:
+   * GstRTPBasePayload:ptime-multiple:
    *
    * Force buffers to be multiples of this duration in ns (0 disables)
    **/
@@ -239,10 +265,67 @@ gst_rtp_base_payload_class_init (GstRTPBasePayloadClass * klass)
           0, G_MAXINT64, DEFAULT_PTIME_MULTIPLE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GstRTPBasePayload:stats:
+   *
+   * Various payloader statistics retrieved atomically (and are therefore
+   * synchroized with each other), these can be used e.g. to generate an
+   * RTP-Info header. This property return a GstStructure named
+   * application/x-rtp-payload-stats containing the following fields relating to
+   * the last processed buffer and current state of the stream being payloaded:
+   *
+   * <variablelist>
+   *   <varlistentry>
+   *     <term>clock-rate</term>
+   *     <listitem><para>#G_TYPE_UINT, clock-rate of the
+   *     stream</para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>running-time</term>
+   *     <listitem><para>#G_TYPE_UINT64, running time
+   *     </para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>seqnum</term>
+   *     <listitem><para>#G_TYPE_UINT, sequence number, same as
+   *     #GstRTPBasePayload:seqnum</para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>timestamp</term>
+   *     <listitem><para>#G_TYPE_UINT, RTP timestamp, same as
+   *     #GstRTPBasePayload:timestamp</para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>ssrc</term>
+   *     <listitem><para>#G_TYPE_UINT, The SSRC in use
+   *     </para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>pt</term>
+   *     <listitem><para>#G_TYPE_UINT, The Payload type in use, same as
+   *     #GstRTPBasePayload:pt</para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>seqnum-offset</term>
+   *     <listitem><para>#G_TYPE_UINT, The current offset added to the
+   *     seqnum</para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>timestamp-offset</term>
+   *     <listitem><para>#G_TYPE_UINT, The current offset added to the
+   *     timestamp</para></listitem>
+   *   </varlistentry>
+   * </variablelist>
+   **/
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_STATS,
+      g_param_spec_boxed ("stats", "Statistics", "Various statistics",
+          GST_TYPE_STRUCTURE, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   gstelement_class->change_state = gst_rtp_base_payload_change_state;
 
   klass->get_caps = gst_rtp_base_payload_getcaps_default;
   klass->sink_event = gst_rtp_base_payload_sink_event_default;
+  klass->src_event = gst_rtp_base_payload_src_event_default;
   klass->query = gst_rtp_base_payload_query_default;
 
   GST_DEBUG_CATEGORY_INIT (rtpbasepayload_debug, "rtpbasepayload", 0,
@@ -263,6 +346,8 @@ gst_rtp_base_payload_init (GstRTPBasePayload * rtpbasepayload, gpointer g_class)
   g_return_if_fail (templ != NULL);
 
   rtpbasepayload->srcpad = gst_pad_new_from_template (templ, "src");
+  gst_pad_set_event_function (rtpbasepayload->srcpad,
+      gst_rtp_base_payload_src_event);
   gst_element_add_pad (GST_ELEMENT (rtpbasepayload), rtpbasepayload->srcpad);
 
   templ =
@@ -283,16 +368,18 @@ gst_rtp_base_payload_init (GstRTPBasePayload * rtpbasepayload, gpointer g_class)
   rtpbasepayload->seqnum_offset = DEFAULT_SEQNUM_OFFSET;
   rtpbasepayload->ssrc = DEFAULT_SSRC;
   rtpbasepayload->ts_offset = DEFAULT_TIMESTAMP_OFFSET;
+  priv->running_time = DEFAULT_RUNNING_TIME;
   priv->seqnum_offset_random = (rtpbasepayload->seqnum_offset == -1);
   priv->ts_offset_random = (rtpbasepayload->ts_offset == -1);
   priv->ssrc_random = (rtpbasepayload->ssrc == -1);
+  priv->pt_set = FALSE;
 
   rtpbasepayload->max_ptime = DEFAULT_MAX_PTIME;
   rtpbasepayload->min_ptime = DEFAULT_MIN_PTIME;
   rtpbasepayload->priv->perfect_rtptime = DEFAULT_PERFECT_RTPTIME;
   rtpbasepayload->ptime_multiple = DEFAULT_PTIME_MULTIPLE;
   rtpbasepayload->priv->base_offset = GST_BUFFER_OFFSET_NONE;
-  rtpbasepayload->priv->base_rtime = GST_BUFFER_OFFSET_NONE;
+  rtpbasepayload->priv->base_rtime_hz = GST_BUFFER_OFFSET_NONE;
 
   rtpbasepayload->media = NULL;
   rtpbasepayload->encoding_name = NULL;
@@ -315,6 +402,8 @@ gst_rtp_base_payload_finalize (GObject * object)
   g_free (rtpbasepayload->encoding_name);
   rtpbasepayload->encoding_name = NULL;
 
+  gst_caps_replace (&rtpbasepayload->priv->subclass_srccaps, NULL);
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -365,7 +454,7 @@ gst_rtp_base_payload_sink_event_default (GstRTPBasePayload * rtpbasepayload,
       if (rtpbasepayload_class->set_caps)
         res = rtpbasepayload_class->set_caps (rtpbasepayload, caps);
       else
-        res = TRUE;
+        res = gst_rtp_base_payload_negotiate (rtpbasepayload);
 
       rtpbasepayload->priv->negotiated = res;
 
@@ -385,6 +474,7 @@ gst_rtp_base_payload_sink_event_default (GstRTPBasePayload * rtpbasepayload,
           "configured SEGMENT %" GST_SEGMENT_FORMAT, segment);
       if (rtpbasepayload->priv->delay_segment) {
         gst_event_replace (&rtpbasepayload->priv->pending_segment, event);
+        gst_event_unref (event);
         res = TRUE;
       } else {
         res = gst_pad_event_default (rtpbasepayload->sinkpad, parent, event);
@@ -417,6 +507,83 @@ gst_rtp_base_payload_sink_event (GstPad * pad, GstObject * parent,
   return res;
 }
 
+static gboolean
+gst_rtp_base_payload_src_event_default (GstRTPBasePayload * rtpbasepayload,
+    GstEvent * event)
+{
+  GstObject *parent = GST_OBJECT_CAST (rtpbasepayload);
+  gboolean res = TRUE, forward = TRUE;
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_CUSTOM_UPSTREAM:
+    {
+      const GstStructure *s = gst_event_get_structure (event);
+
+      if (gst_structure_has_name (s, "GstRTPCollision")) {
+        guint ssrc = 0;
+
+        if (!gst_structure_get_uint (s, "ssrc", &ssrc))
+          ssrc = -1;
+
+        GST_DEBUG_OBJECT (rtpbasepayload, "collided ssrc: %" G_GUINT32_FORMAT,
+            ssrc);
+
+        /* choose another ssrc for our stream */
+        if (ssrc == rtpbasepayload->current_ssrc) {
+          GstCaps *caps;
+          guint suggested_ssrc = 0;
+
+          if (gst_structure_get_uint (s, "suggested-ssrc", &suggested_ssrc))
+            rtpbasepayload->current_ssrc = suggested_ssrc;
+
+          while (ssrc == rtpbasepayload->current_ssrc)
+            rtpbasepayload->current_ssrc = g_random_int ();
+
+          caps = gst_pad_get_current_caps (rtpbasepayload->srcpad);
+          caps = gst_caps_make_writable (caps);
+          gst_caps_set_simple (caps,
+              "ssrc", G_TYPE_UINT, rtpbasepayload->current_ssrc, NULL);
+          res = gst_pad_set_caps (rtpbasepayload->srcpad, caps);
+          gst_caps_unref (caps);
+
+          /* the event was for us */
+          forward = FALSE;
+        }
+      }
+      break;
+    }
+    default:
+      break;
+  }
+
+  if (forward)
+    res = gst_pad_event_default (rtpbasepayload->srcpad, parent, event);
+  else
+    gst_event_unref (event);
+
+  return res;
+}
+
+static gboolean
+gst_rtp_base_payload_src_event (GstPad * pad, GstObject * parent,
+    GstEvent * event)
+{
+  GstRTPBasePayload *rtpbasepayload;
+  GstRTPBasePayloadClass *rtpbasepayload_class;
+  gboolean res = FALSE;
+
+  rtpbasepayload = GST_RTP_BASE_PAYLOAD (parent);
+  rtpbasepayload_class = GST_RTP_BASE_PAYLOAD_GET_CLASS (rtpbasepayload);
+
+  if (rtpbasepayload_class->src_event)
+    res = rtpbasepayload_class->src_event (rtpbasepayload, event);
+  else
+    gst_event_unref (event);
+
+  return res;
+}
+
+
 static gboolean
 gst_rtp_base_payload_query_default (GstRTPBasePayload * rtpbasepayload,
     GstPad * pad, GstQuery * query)
@@ -483,6 +650,9 @@ gst_rtp_base_payload_chain (GstPad * pad, GstObject * parent,
   if (!rtpbasepayload->priv->negotiated)
     goto not_negotiated;
 
+  if (gst_pad_check_reconfigure (GST_RTP_BASE_PAYLOAD_SRCPAD (rtpbasepayload)))
+    gst_rtp_base_payload_negotiate (rtpbasepayload);
+
   ret = rtpbasepayload_class->handle_buffer (rtpbasepayload, buffer);
 
   return ret;
@@ -574,8 +744,7 @@ gboolean
 gst_rtp_base_payload_set_outcaps (GstRTPBasePayload * payload,
     const gchar * fieldname, ...)
 {
-  GstCaps *srccaps, *peercaps;
-  gboolean res;
+  GstCaps *srccaps;
 
   /* fill in the defaults, their properties cannot be negotiated. */
   srccaps = gst_caps_new_simple ("application/x-rtp",
@@ -596,13 +765,38 @@ gst_rtp_base_payload_set_outcaps (GstRTPBasePayload * payload,
     GST_DEBUG_OBJECT (payload, "custom added: %" GST_PTR_FORMAT, srccaps);
   }
 
+  gst_caps_replace (&payload->priv->subclass_srccaps, srccaps);
+  gst_caps_unref (srccaps);
+
+  return gst_rtp_base_payload_negotiate (payload);
+}
+
+static gboolean
+gst_rtp_base_payload_negotiate (GstRTPBasePayload * payload)
+{
+  GstCaps *templ, *peercaps, *srccaps;
+  gboolean res;
+
   payload->priv->caps_max_ptime = DEFAULT_MAX_PTIME;
   payload->ptime = 0;
 
-  /* the peer caps can override some of the defaults */
-  peercaps = gst_pad_peer_query_caps (payload->srcpad, srccaps);
+  gst_pad_check_reconfigure (payload->srcpad);
+
+  templ = gst_pad_get_pad_template_caps (payload->srcpad);
+
+  if (payload->priv->subclass_srccaps) {
+    GstCaps *tmp = gst_caps_intersect (payload->priv->subclass_srccaps,
+        templ);
+    gst_caps_unref (templ);
+    templ = tmp;
+  }
+
+  peercaps = gst_pad_peer_query_caps (payload->srcpad, templ);
+
   if (peercaps == NULL) {
     /* no peer caps, just add the other properties */
+
+    srccaps = gst_caps_copy (templ);
     gst_caps_set_simple (srccaps,
         "payload", G_TYPE_INT, GST_RTP_BASE_PAYLOAD_PT (payload),
         "ssrc", G_TYPE_UINT, payload->current_ssrc,
@@ -614,19 +808,203 @@ gst_rtp_base_payload_set_outcaps (GstRTPBasePayload * payload,
     GstCaps *temp;
     GstStructure *s, *d;
     const GValue *value;
-    gint pt;
+    gboolean have_pt = FALSE;
+    gboolean have_ts_offset = FALSE;
+    gboolean have_seqnum_offset = FALSE;
     guint max_ptime, ptime;
 
     /* peer provides caps we can use to fixate. They are already intersected
      * with our srccaps, just make them writable */
     temp = gst_caps_make_writable (peercaps);
-    gst_caps_unref (srccaps);
+    peercaps = NULL;
 
     if (gst_caps_is_empty (temp)) {
       gst_caps_unref (temp);
+      gst_caps_unref (templ);
       return FALSE;
     }
 
+    /* We prefer the pt, timestamp-offset, seqnum-offset from the
+     * property (if set), or any previously configured value over what
+     * downstream prefers. Only if downstream can't accept that, or the
+     * properties were not set, we fall back to choosing downstream's
+     * preferred value
+     *
+     * For ssrc we prefer any value downstream suggests, otherwise
+     * the property value or as a last resort a random value.
+     * This difference for ssrc is implemented for retaining backwards
+     * compatibility with changing rtpsession's internal-ssrc property.
+     *
+     * FIXME 2.0: All these properties should go away and be negotiated
+     * via caps only!
+     */
+
+    /* try to use the previously set pt, or the one from the property */
+    if (payload->priv->pt_set || gst_pad_has_current_caps (payload->srcpad)) {
+      GstCaps *probe_caps = gst_caps_copy (templ);
+      GstCaps *intersection;
+
+      gst_caps_set_simple (probe_caps, "payload", G_TYPE_INT,
+          GST_RTP_BASE_PAYLOAD_PT (payload), NULL);
+      intersection = gst_caps_intersect (probe_caps, temp);
+
+      if (!gst_caps_is_empty (intersection)) {
+        GST_LOG_OBJECT (payload, "Using selected pt %d",
+            GST_RTP_BASE_PAYLOAD_PT (payload));
+        have_pt = TRUE;
+        gst_caps_unref (temp);
+        temp = intersection;
+      } else {
+        GST_WARNING_OBJECT (payload, "Can't use selected pt %d",
+            GST_RTP_BASE_PAYLOAD_PT (payload));
+        gst_caps_unref (intersection);
+      }
+      gst_caps_unref (probe_caps);
+    }
+
+    /* If we got no pt above, select one now */
+    if (!have_pt) {
+      gint pt;
+
+      /* get first structure */
+      s = gst_caps_get_structure (temp, 0);
+
+      if (gst_structure_get_int (s, "payload", &pt)) {
+        /* use peer pt */
+        GST_RTP_BASE_PAYLOAD_PT (payload) = pt;
+        GST_LOG_OBJECT (payload, "using peer pt %d", pt);
+      } else {
+        if (gst_structure_has_field (s, "payload")) {
+          /* can only fixate if there is a field */
+          gst_structure_fixate_field_nearest_int (s, "payload",
+              GST_RTP_BASE_PAYLOAD_PT (payload));
+          gst_structure_get_int (s, "payload", &pt);
+          GST_RTP_BASE_PAYLOAD_PT (payload) = pt;
+          GST_LOG_OBJECT (payload, "using peer pt %d", pt);
+        } else {
+          /* no pt field, use the internal pt */
+          pt = GST_RTP_BASE_PAYLOAD_PT (payload);
+          gst_structure_set (s, "payload", G_TYPE_INT, pt, NULL);
+          GST_LOG_OBJECT (payload, "using internal pt %d", pt);
+        }
+      }
+      s = NULL;
+    }
+
+    /* If we got no ssrc above, select one now */
+    /* get first structure */
+    s = gst_caps_get_structure (temp, 0);
+
+    if (gst_structure_has_field_typed (s, "ssrc", G_TYPE_UINT)) {
+      value = gst_structure_get_value (s, "ssrc");
+      payload->current_ssrc = g_value_get_uint (value);
+      GST_LOG_OBJECT (payload, "using peer ssrc %08x", payload->current_ssrc);
+    } else {
+      /* FIXME, fixate_nearest_uint would be even better but we
+       * don't support uint ranges so how likely is it that anybody
+       * uses a list of possible ssrcs */
+      gst_structure_set (s, "ssrc", G_TYPE_UINT, payload->current_ssrc, NULL);
+      GST_LOG_OBJECT (payload, "using internal ssrc %08x",
+          payload->current_ssrc);
+    }
+    s = NULL;
+
+    /* try to select the previously used timestamp-offset, or the one from the property */
+    if (!payload->priv->ts_offset_random
+        || gst_pad_has_current_caps (payload->srcpad)) {
+      GstCaps *probe_caps = gst_caps_copy (templ);
+      GstCaps *intersection;
+
+      gst_caps_set_simple (probe_caps, "timestamp-offset", G_TYPE_UINT,
+          payload->ts_base, NULL);
+      intersection = gst_caps_intersect (probe_caps, temp);
+
+      if (!gst_caps_is_empty (intersection)) {
+        GST_LOG_OBJECT (payload, "Using selected timestamp-offset %u",
+            payload->ts_base);
+        gst_caps_unref (temp);
+        temp = intersection;
+        have_ts_offset = TRUE;
+      } else {
+        GST_WARNING_OBJECT (payload, "Can't use selected timestamp-offset %u",
+            payload->ts_base);
+        gst_caps_unref (intersection);
+      }
+      gst_caps_unref (probe_caps);
+    }
+
+    /* If we got no timestamp-offset above, select one now */
+    if (!have_ts_offset) {
+      /* get first structure */
+      s = gst_caps_get_structure (temp, 0);
+
+      if (gst_structure_has_field_typed (s, "timestamp-offset", G_TYPE_UINT)) {
+        value = gst_structure_get_value (s, "timestamp-offset");
+        payload->ts_base = g_value_get_uint (value);
+        GST_LOG_OBJECT (payload, "using peer timestamp-offset %u",
+            payload->ts_base);
+      } else {
+        /* FIXME, fixate_nearest_uint would be even better but we
+         * don't support uint ranges so how likely is it that anybody
+         * uses a list of possible timestamp-offsets */
+        gst_structure_set (s, "timestamp-offset", G_TYPE_UINT, payload->ts_base,
+            NULL);
+        GST_LOG_OBJECT (payload, "using internal timestamp-offset %u",
+            payload->ts_base);
+      }
+      s = NULL;
+    }
+
+    /* try to select the previously used seqnum-offset, or the one from the property */
+    if (!payload->priv->seqnum_offset_random
+        || gst_pad_has_current_caps (payload->srcpad)) {
+      GstCaps *probe_caps = gst_caps_copy (templ);
+      GstCaps *intersection;
+
+      gst_caps_set_simple (probe_caps, "seqnum-offset", G_TYPE_UINT,
+          payload->seqnum_base, NULL);
+      intersection = gst_caps_intersect (probe_caps, temp);
+
+      if (!gst_caps_is_empty (intersection)) {
+        GST_LOG_OBJECT (payload, "Using selected seqnum-offset %u",
+            payload->seqnum_base);
+        gst_caps_unref (temp);
+        temp = intersection;
+        have_seqnum_offset = TRUE;
+      } else {
+        GST_WARNING_OBJECT (payload, "Can't use selected seqnum-offset %u",
+            payload->seqnum_base);
+        gst_caps_unref (intersection);
+      }
+      gst_caps_unref (probe_caps);
+    }
+
+    /* If we got no seqnum-offset above, select one now */
+    if (!have_seqnum_offset) {
+      /* get first structure */
+      s = gst_caps_get_structure (temp, 0);
+
+      if (gst_structure_has_field_typed (s, "seqnum-offset", G_TYPE_UINT)) {
+        value = gst_structure_get_value (s, "seqnum-offset");
+        payload->seqnum_base = g_value_get_uint (value);
+        GST_LOG_OBJECT (payload, "using peer seqnum-offset %u",
+            payload->seqnum_base);
+        payload->priv->next_seqnum = payload->seqnum_base;
+        payload->seqnum = payload->seqnum_base;
+        payload->priv->seqnum_offset_random = FALSE;
+      } else {
+        /* FIXME, fixate_nearest_uint would be even better but we
+         * don't support uint ranges so how likely is it that anybody
+         * uses a list of possible seqnum-offsets */
+        gst_structure_set (s, "seqnum-offset", G_TYPE_UINT,
+            payload->seqnum_base, NULL);
+        GST_LOG_OBJECT (payload, "using internal seqnum-offset %u",
+            payload->seqnum_base);
+      }
+
+      s = NULL;
+    }
+
     /* now fixate, start by taking the first caps */
     temp = gst_caps_truncate (temp);
 
@@ -639,63 +1017,8 @@ gst_rtp_base_payload_set_outcaps (GstRTPBasePayload * payload,
     if (gst_structure_get_uint (s, "ptime", &ptime))
       payload->ptime = ptime * GST_MSECOND;
 
-    if (gst_structure_get_int (s, "payload", &pt)) {
-      /* use peer pt */
-      GST_RTP_BASE_PAYLOAD_PT (payload) = pt;
-      GST_LOG_OBJECT (payload, "using peer pt %d", pt);
-    } else {
-      if (gst_structure_has_field (s, "payload")) {
-        /* can only fixate if there is a field */
-        gst_structure_fixate_field_nearest_int (s, "payload",
-            GST_RTP_BASE_PAYLOAD_PT (payload));
-        gst_structure_get_int (s, "payload", &pt);
-        GST_LOG_OBJECT (payload, "using peer pt %d", pt);
-      } else {
-        /* no pt field, use the internal pt */
-        pt = GST_RTP_BASE_PAYLOAD_PT (payload);
-        gst_structure_set (s, "payload", G_TYPE_INT, pt, NULL);
-        GST_LOG_OBJECT (payload, "using internal pt %d", pt);
-      }
-    }
-
-    if (gst_structure_has_field_typed (s, "ssrc", G_TYPE_UINT)) {
-      value = gst_structure_get_value (s, "ssrc");
-      payload->current_ssrc = g_value_get_uint (value);
-      GST_LOG_OBJECT (payload, "using peer ssrc %08x", payload->current_ssrc);
-    } else {
-      /* FIXME, fixate_nearest_uint would be even better */
-      gst_structure_set (s, "ssrc", G_TYPE_UINT, payload->current_ssrc, NULL);
-      GST_LOG_OBJECT (payload, "using internal ssrc %08x",
-          payload->current_ssrc);
-    }
-
-    if (gst_structure_has_field_typed (s, "timestamp-offset", G_TYPE_UINT)) {
-      value = gst_structure_get_value (s, "timestamp-offset");
-      payload->ts_base = g_value_get_uint (value);
-      GST_LOG_OBJECT (payload, "using peer timestamp-offset %u",
-          payload->ts_base);
-    } else {
-      /* FIXME, fixate_nearest_uint would be even better */
-      gst_structure_set (s, "timestamp-offset", G_TYPE_UINT, payload->ts_base,
-          NULL);
-      GST_LOG_OBJECT (payload, "using internal timestamp-offset %u",
-          payload->ts_base);
-    }
-    if (gst_structure_has_field_typed (s, "seqnum-offset", G_TYPE_UINT)) {
-      value = gst_structure_get_value (s, "seqnum-offset");
-      payload->seqnum_base = g_value_get_uint (value);
-      GST_LOG_OBJECT (payload, "using peer seqnum-offset %u",
-          payload->seqnum_base);
-    } else {
-      /* FIXME, fixate_nearest_uint would be even better */
-      gst_structure_set (s, "seqnum-offset", G_TYPE_UINT, payload->seqnum_base,
-          NULL);
-      GST_LOG_OBJECT (payload, "using internal seqnum-offset %u",
-          payload->seqnum_base);
-    }
-
-    /* make the target caps by copying over all the fixed caps, removing the
-     * unfixed caps. */
+    /* make the target caps by copying over all the fixed fields, removing the
+     * unfixed fields. */
     srccaps = gst_caps_new_empty_simple (gst_structure_get_name (s));
     d = gst_caps_get_structure (srccaps, 0);
 
@@ -710,6 +1033,7 @@ gst_rtp_base_payload_set_outcaps (GstRTPBasePayload * payload,
 
   res = gst_pad_set_caps (GST_RTP_BASE_PAYLOAD_SRCPAD (payload), srccaps);
   gst_caps_unref (srccaps);
+  gst_caps_unref (templ);
 
   return res;
 }
@@ -833,32 +1157,55 @@ gst_rtp_base_payload_prepare_push (GstRTPBasePayload * payload,
   /* convert to RTP time */
   if (priv->perfect_rtptime && data.offset != GST_BUFFER_OFFSET_NONE &&
       priv->base_offset != GST_BUFFER_OFFSET_NONE) {
-    /* if we have an offset, use that for making an RTP timestamp */
-    data.rtptime = payload->ts_base + priv->base_rtime +
-        data.offset - priv->base_offset;
+    /* generate perfect RTP time by adding together the base timestamp, the
+     * running time of the first buffer and difference between the offset of the
+     * first buffer and the offset of the current buffer. */
+    guint64 offset = data.offset - priv->base_offset;
+    data.rtptime = payload->ts_base + priv->base_rtime_hz + offset;
+
     GST_LOG_OBJECT (payload,
         "Using offset %" G_GUINT64_FORMAT " for RTP timestamp", data.offset);
+
+    /* store buffer's running time */
+    GST_LOG_OBJECT (payload,
+        "setting running-time to %" G_GUINT64_FORMAT,
+        data.offset - priv->base_offset);
+    priv->running_time = priv->base_rtime + data.offset - priv->base_offset;
   } else if (GST_CLOCK_TIME_IS_VALID (data.pts)) {
-    gint64 rtime;
+    guint64 rtime_ns;
+    guint64 rtime_hz;
 
     /* no offset, use the gstreamer pts */
-    rtime = gst_segment_to_running_time (&payload->segment, GST_FORMAT_TIME,
+    rtime_ns = gst_segment_to_running_time (&payload->segment, GST_FORMAT_TIME,
         data.pts);
 
-    if (rtime == -1) {
+    if (!GST_CLOCK_TIME_IS_VALID (rtime_ns)) {
       GST_LOG_OBJECT (payload, "Clipped pts, using base RTP timestamp");
-      rtime = 0;
+      rtime_hz = 0;
     } else {
       GST_LOG_OBJECT (payload,
           "Using running_time %" GST_TIME_FORMAT " for RTP timestamp",
-          GST_TIME_ARGS (rtime));
-      rtime =
-          gst_util_uint64_scale_int (rtime, payload->clock_rate, GST_SECOND);
+          GST_TIME_ARGS (rtime_ns));
+      rtime_hz =
+          gst_util_uint64_scale_int (rtime_ns, payload->clock_rate, GST_SECOND);
       priv->base_offset = data.offset;
-      priv->base_rtime = rtime;
+      priv->base_rtime_hz = rtime_hz;
     }
+
     /* add running_time in clock-rate units to the base timestamp */
-    data.rtptime = payload->ts_base + rtime;
+    data.rtptime = payload->ts_base + rtime_hz;
+
+    /* store buffer's running time */
+    if (priv->perfect_rtptime) {
+      GST_LOG_OBJECT (payload,
+          "setting running-time to %" G_GUINT64_FORMAT, rtime_hz);
+      priv->running_time = rtime_hz;
+    } else {
+      GST_LOG_OBJECT (payload,
+          "setting running-time to %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (rtime_ns));
+      priv->running_time = rtime_ns;
+    }
   } else {
     GST_LOG_OBJECT (payload,
         "Using previous RTP timestamp %" G_GUINT32_FORMAT, payload->timestamp);
@@ -882,8 +1229,8 @@ gst_rtp_base_payload_prepare_push (GstRTPBasePayload * payload,
       (is_list) ? -1 : gst_buffer_get_size (GST_BUFFER (obj)),
       payload->seqnum, data.rtptime, GST_TIME_ARGS (data.pts));
 
-  if (g_atomic_int_compare_and_exchange (&payload->priv->
-          notified_first_timestamp, 1, 0)) {
+  if (g_atomic_int_compare_and_exchange (&payload->
+          priv->notified_first_timestamp, 1, 0)) {
     g_object_notify (G_OBJECT (payload), "timestamp");
     g_object_notify (G_OBJECT (payload), "seqnum");
   }
@@ -966,6 +1313,27 @@ gst_rtp_base_payload_push (GstRTPBasePayload * payload, GstBuffer * buffer)
   return res;
 }
 
+static GstStructure *
+gst_rtp_base_payload_create_stats (GstRTPBasePayload * rtpbasepayload)
+{
+  GstRTPBasePayloadPrivate *priv;
+  GstStructure *s;
+
+  priv = rtpbasepayload->priv;
+
+  s = gst_structure_new ("application/x-rtp-payload-stats",
+      "clock-rate", G_TYPE_UINT, (guint) rtpbasepayload->clock_rate,
+      "running-time", G_TYPE_UINT64, priv->running_time,
+      "seqnum", G_TYPE_UINT, (guint) rtpbasepayload->seqnum,
+      "timestamp", G_TYPE_UINT, (guint) rtpbasepayload->timestamp,
+      "ssrc", G_TYPE_UINT, rtpbasepayload->current_ssrc,
+      "pt", G_TYPE_UINT, rtpbasepayload->pt,
+      "seqnum-offset", G_TYPE_UINT, (guint) rtpbasepayload->seqnum_base,
+      "timestamp-offset", G_TYPE_UINT, (guint) rtpbasepayload->ts_base, NULL);
+
+  return s;
+}
+
 static void
 gst_rtp_base_payload_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -983,6 +1351,7 @@ gst_rtp_base_payload_set_property (GObject * object, guint prop_id,
       break;
     case PROP_PT:
       rtpbasepayload->pt = g_value_get_uint (value);
+      priv->pt_set = TRUE;
       break;
     case PROP_SSRC:
       val = g_value_get_uint (value);
@@ -1073,6 +1442,10 @@ gst_rtp_base_payload_get_property (GObject * object, guint prop_id,
     case PROP_PTIME_MULTIPLE:
       g_value_set_int64 (value, rtpbasepayload->ptime_multiple);
       break;
+    case PROP_STATS:
+      g_value_take_boxed (value,
+          gst_rtp_base_payload_create_stats (rtpbasepayload));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1099,7 +1472,7 @@ gst_rtp_base_payload_change_state (GstElement * element,
       gst_event_replace (&rtpbasepayload->priv->pending_segment, NULL);
 
       if (priv->seqnum_offset_random)
-        rtpbasepayload->seqnum_base = g_random_int_range (0, G_MAXUINT16);
+        rtpbasepayload->seqnum_base = g_random_int_range (0, G_MAXINT16);
       else
         rtpbasepayload->seqnum_base = rtpbasepayload->seqnum_offset;
       priv->next_seqnum = rtpbasepayload->seqnum_base;
@@ -1115,9 +1488,11 @@ gst_rtp_base_payload_change_state (GstElement * element,
       else
         rtpbasepayload->ts_base = rtpbasepayload->ts_offset;
       rtpbasepayload->timestamp = rtpbasepayload->ts_base;
+      priv->running_time = DEFAULT_RUNNING_TIME;
       g_atomic_int_set (&rtpbasepayload->priv->notified_first_timestamp, 1);
       priv->base_offset = GST_BUFFER_OFFSET_NONE;
       priv->negotiated = FALSE;
+      gst_caps_replace (&rtpbasepayload->priv->subclass_srccaps, NULL);
       break;
     default:
       break;
