@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -469,7 +469,7 @@ MAKE_PACK_FUNC_ORC_FI_S (s32_be_float, 4, WRITE32_TO_BE);
 #define audio_convert_pack_double_hq_le MAKE_UNPACK_FUNC_NAME (double_hq_le)
 #define audio_convert_pack_double_hq_be MAKE_UNPACK_FUNC_NAME (double_hq_be)
 
-static AudioConvertUnpack unpack_funcs[] = {
+static const AudioConvertUnpack unpack_funcs[] = {
   (AudioConvertUnpack) MAKE_UNPACK_FUNC_NAME (u8),
   (AudioConvertUnpack) MAKE_UNPACK_FUNC_NAME (s8),
   (AudioConvertUnpack) MAKE_UNPACK_FUNC_NAME (u8),
@@ -512,7 +512,7 @@ static AudioConvertUnpack unpack_funcs[] = {
   (AudioConvertUnpack) MAKE_UNPACK_FUNC_NAME (s32_be_float),
 };
 
-static AudioConvertPack pack_funcs[] = {
+static const AudioConvertPack pack_funcs[] = {
   (AudioConvertPack) MAKE_PACK_FUNC_NAME (u8),
   (AudioConvertPack) MAKE_PACK_FUNC_NAME (s8),
   (AudioConvertPack) MAKE_PACK_FUNC_NAME (u8),
