@@ -38,32 +38,12 @@ gst_fast_random_uint32 (void)
   return (state = state * 1103515245 + 12345);
 }
 
-static inline guint32
-gst_fast_random_uint32_range (gint32 start, gint32 end)
-{
-  guint64 tmp = gst_fast_random_uint32 ();
-
-  tmp = (tmp * (end - start)) / G_MAXUINT32 + start;
-
-  return (guint32) tmp;
-}
-
 static inline gint32
 gst_fast_random_int32 (void)
 {
   return (gint32) gst_fast_random_uint32 ();
 }
 
-static inline gint32
-gst_fast_random_int32_range (gint32 start, gint32 end)
-{
-  gint64 tmp = gst_fast_random_uint32 ();
-
-  tmp = (tmp * (end - start)) / G_MAXUINT32 + start;
-
-  return (gint32) tmp;
-}
-
 static inline gdouble
 gst_fast_random_double (void)
 {
